@@ -8,13 +8,12 @@ use File::Basename qw(dirname);
 use File::Path qw(mkpath);
 use CPAN::FTP::netrc;
 use vars qw($connect_to_internet_ok $Ua $Thesite $ThesiteURL $Themethod);
-
 @CPAN::FTP::ISA = qw(CPAN::Debug);
 
 use vars qw(
             $VERSION
 );
-$VERSION = "5.5004";
+$VERSION = "5.5";
 
 #-> sub CPAN::FTP::ftp_statistics
 # if they want to rewrite, they need to pass in a filehandle
@@ -43,7 +42,7 @@ sub _ftp_statistics {
     if ($@) {
         if (ref $@) {
             if (ref $@ eq "CPAN::Exception::yaml_not_installed") {
-                $CPAN::Frontend->myprint("Warning (usually harmless): $@\n");
+                $CPAN::Frontend->myprint("Warning (usually harmless): $@");
                 return;
             } elsif (ref $@ eq "CPAN::Exception::yaml_process_error") {
                 $CPAN::Frontend->mydie($@);
@@ -119,7 +118,7 @@ sub _add_to_statistics {
                                ));
         }
         # Win32 cannot rename a file to an existing filename
-        unlink($sfile) if ($^O eq 'MSWin32' or $^O eq 'os2');
+        unlink($sfile) if ($^O eq 'MSWin32');
 	_copy_stat($sfile, "$sfile.$$") if -e $sfile;
         rename "$sfile.$$", $sfile
             or $CPAN::Frontend->mydie("Could not rename '$sfile.$$' to '$sfile': $!\n");
@@ -160,7 +159,8 @@ sub _copy_stat {
 # checked from, maybe only for young files?
 #-> sub CPAN::FTP::_recommend_url_for
 sub _recommend_url_for {
-    my($self, $file, $urllist) = @_;
+    my($self, $file) = @_;
+    my $urllist = $self->_get_urllist;
     if ($file =~ s|/CHECKSUMS(.gz)?$||) {
         my $fullstats = $self->_ftp_statistics();
         my $history = $fullstats->{history} || [];
@@ -183,16 +183,13 @@ sub _recommend_url_for {
 
 #-> sub CPAN::FTP::_get_urllist
 sub _get_urllist {
-    my($self, $with_defaults) = @_;
-    CPAN->debug("with_defaults[$with_defaults]") if $CPAN::DEBUG;
-
+    my($self) = @_;
     $CPAN::Config->{urllist} ||= [];
     unless (ref $CPAN::Config->{urllist} eq 'ARRAY') {
         $CPAN::Frontend->mywarn("Malformed urllist; ignoring.  Configuration file corrupt?\n");
         $CPAN::Config->{urllist} = [];
     }
     my @urllist = grep { defined $_ and length $_ } @{$CPAN::Config->{urllist}};
-    push @urllist, @CPAN::Defaultsites if $with_defaults;
     for my $u (@urllist) {
         CPAN->debug("u[$u]") if $CPAN::DEBUG;
         if (UNIVERSAL::can($u,"text")) {
@@ -222,19 +219,19 @@ sub ftp_get {
     $class->debug(qq[Going to login("anonymous","$Config::Config{cf_email}")]);
     unless ( $ftp->login("anonymous",$Config::Config{'cf_email'}) ) {
         my $msg = $ftp->message;
-        $CPAN::Frontend->mywarn("  Couldn't login on $host: $msg\n");
+        $CPAN::Frontend->mywarn("  Couldn't login on $host: $msg");
         return;
     }
     unless ( $ftp->cwd($dir) ) {
         my $msg = $ftp->message;
-        $CPAN::Frontend->mywarn("  Couldn't cwd $dir: $msg\n");
+        $CPAN::Frontend->mywarn("  Couldn't cwd $dir: $msg");
         return;
     }
     $ftp->binary;
     $class->debug(qq[Going to ->get("$file","$target")\n]) if $CPAN::DEBUG;
     unless ( $ftp->get($file,$target) ) {
         my $msg = $ftp->message;
-        $CPAN::Frontend->mywarn("  Couldn't fetch $file from $host: $msg\n");
+        $CPAN::Frontend->mywarn("  Couldn't fetch $file from $host: $msg");
         return;
     }
     $ftp->quit; # it's ok if this fails
@@ -271,9 +268,9 @@ sub ftp_get {
 
 #-> sub CPAN::FTP::localize ;
 sub localize {
-    my($self,$file,$aslocal,$force,$with_defaults) = @_;
+    my($self,$file,$aslocal,$force) = @_;
     $force ||= 0;
-    Carp::croak( "Usage: ->localize(cpan_file,as_local_file[,\$force])" )
+    Carp::croak( "Usage: ->localize(cpan_file,as_local_file[,$force])" )
         unless defined $aslocal;
     if ($CPAN::DEBUG){
         require Carp;
@@ -349,7 +346,7 @@ sub localize {
     # Try the list of urls for each single object. We keep a record
     # where we did get a file from
     my(@reordered,$last);
-    my $ccurllist = $self->_get_urllist($with_defaults);
+    my $ccurllist = $self->_get_urllist;
     $last = $#$ccurllist;
     if ($force & 2) { # local cpans probably out of date, don't reorder
         @reordered = (0..$last);
@@ -437,7 +434,7 @@ I would like to connect to one of the following sites to get '%s':
         }
         $self->debug("synth. urllist[@urllist]") if $CPAN::DEBUG;
         my $aslocal_tempfile = $aslocal . ".tmp" . $$;
-        if (my $recommend = $self->_recommend_url_for($file,\@urllist)) {
+        if (my $recommend = $self->_recommend_url_for($file)) {
             @urllist = grep { $_ ne $recommend } @urllist;
             unshift @urllist, $recommend;
         }
@@ -453,10 +450,6 @@ I would like to connect to one of the following sites to get '%s':
                                               "'$ret' to '$aslocal': $!");
                 $ret = $aslocal;
             }
-            elsif (-f $ret && $scheme eq 'file' ) {
-                # it's a local file, so there's nothing left to do, we
-                # let them read from where it is
-            }
             $Themethod = $level;
             my $now = time;
             # utime $now, $now, $aslocal; # too bad, if we do that, we
@@ -492,12 +485,13 @@ I would like to connect to one of the following sites to get '%s':
         push @mess, qq{The urllist can be edited.},
             qq{E.g. with 'o conf urllist push ftp://myurl/'};
         $CPAN::Frontend->mywarn(Text::Wrap::wrap("","","@mess"). "\n\n");
-        $CPAN::Frontend->mydie("Could not fetch $file\n");
+        $CPAN::Frontend->mywarn("Could not fetch $file\n");
+        $CPAN::Frontend->mysleep(2);
     }
     if ($maybe_restore) {
         rename "$aslocal.bak$$", $aslocal;
         $CPAN::Frontend->myprint("Trying to get away with old file:\n" .
-                                 $self->ls($aslocal) . "\n");
+                                 $self->ls($aslocal));
         return $aslocal;
     }
     return;
@@ -543,7 +537,7 @@ sub hostdleasy { #called from hostdlxxx
             my $l;
             if ($CPAN::META->has_inst('URI::URL')) {
                 my $u =  URI::URL->new($url);
-                $l = $u->file;
+                $l = $u->path;
             } else { # works only on Unix, is poorly constructed, but
                 # hopefully better than nothing.
                 # RFC 1738 says fileurl BNF is
@@ -562,53 +556,29 @@ sub hostdleasy { #called from hostdlxxx
                 $ThesiteURL = $ro_url;
                 return $l;
             }
-            # If request is for a compressed file and we can find the 
-            # uncompressed file also, return the path of the uncompressed file
-            # otherwise, decompress it and return the resulting path
             if ($l =~ /(.+)\.gz$/) {
                 my $ungz = $1;
                 if ( -f $ungz && -r _) {
                     $ThesiteURL = $ro_url;
                     return $ungz;
                 }
-                else {
-                    eval { CPAN::Tarzip->new($l)->gunzip($aslocal) };
-                    if ( -f $aslocal) {
-                        $ThesiteURL = $ro_url;
-                        return $aslocal;
-                    }
-                    else {
-                        $CPAN::Frontend->mywarn("Error decompressing '$l': $@\n")
-                            if $@;
-                        return;
-                    }
-                }
             }
-            # Otherwise, return the local file path if it exists
-            elsif ( -f $l && -r _) {
-                $ThesiteURL = $ro_url;
-                return $l;
-            }
-            # If we can't find it, but there is a compressed version
-            # of it, then decompress it
-            elsif (-f "$l.gz") {
+            # Maybe mirror has compressed it?
+            if (-f "$l.gz") {
                 $self->debug("found compressed $l.gz") if $CPAN::DEBUG;
                 eval { CPAN::Tarzip->new("$l.gz")->gunzip($aslocal) };
                 if ( -f $aslocal) {
                     $ThesiteURL = $ro_url;
                     return $aslocal;
                 }
-                else {
-                    $CPAN::Frontend->mywarn("Error decompressing '$l': $@\n")
-                        if $@;
-                    return;
-                }
             }
             $CPAN::Frontend->mywarn("Could not find '$l'\n");
         }
         $self->debug("it was not a file URL") if $CPAN::DEBUG;
         if ($CPAN::META->has_usable('LWP')) {
-            $CPAN::Frontend->myprint("Fetching with LWP:\n$url\n");
+            $CPAN::Frontend->myprint("Fetching with LWP:
+  $url
+");
             unless ($Ua) {
                 CPAN::LWP::UserAgent->config;
                 eval { $Ua = CPAN::LWP::UserAgent->new; };
@@ -626,7 +596,9 @@ sub hostdleasy { #called from hostdlxxx
                 return $aslocal;
             } elsif ($url !~ /\.gz(?!\n)\Z/) {
                 my $gzurl = "$url.gz";
-                $CPAN::Frontend->myprint("Fetching with LWP:\n$gzurl\n");
+                $CPAN::Frontend->myprint("Fetching with LWP:
+  $gzurl
+");
                 $res = $Ua->mirror($gzurl, "$aslocal.gz");
                 if ($res->is_success) {
                     if (eval {CPAN::Tarzip->new("$aslocal.gz")->gunzip($aslocal)}) {
@@ -654,7 +626,9 @@ sub hostdleasy { #called from hostdlxxx
             my($host,$dir,$getfile) = ($1,$2,$3);
             if ($CPAN::META->has_usable('Net::FTP')) {
                 $dir =~ s|/+|/|g;
-                $CPAN::Frontend->myprint("Fetching with Net::FTP:\n$url\n");
+                $CPAN::Frontend->myprint("Fetching with Net::FTP:
+  $url
+");
                 $self->debug("getfile[$getfile]dir[$dir]host[$host]" .
                              "aslocal[$aslocal]") if $CPAN::DEBUG;
                 if (CPAN::FTP->ftp_get($host,$dir,$getfile,$aslocal)) {
@@ -663,7 +637,9 @@ sub hostdleasy { #called from hostdlxxx
                 }
                 if ($aslocal !~ /\.gz(?!\n)\Z/) {
                     my $gz = "$aslocal.gz";
-                    $CPAN::Frontend->myprint("Fetching with Net::FTP\n$url.gz\n");
+                    $CPAN::Frontend->myprint("Fetching with Net::FTP
+  $url.gz
+");
                     if (CPAN::FTP->ftp_get($host,
                                            $dir,
                                            "$getfile.gz",
@@ -706,9 +682,7 @@ sub hostdlhard {
     # < /dev/null ";
     my($aslocal_dir) = dirname($aslocal);
     mkpath($aslocal_dir);
-    my $some_dl_success = 0;
-    my $any_attempt = 0;
- HOSTHARD: for $ro_url (@$host_seq) {
+  HOSTHARD: for $ro_url (@$host_seq) {
         $self->_set_attempt($stats,"dlhard",$ro_url);
         my $url = "$ro_url$file";
         my($proto,$host,$dir,$getfile);
@@ -725,9 +699,6 @@ sub hostdlhard {
         next HOSTHARD if $proto eq "file"; # file URLs would have had
                                            # success above. Likely a bogus URL
 
-        # making at least one attempt against a host
-        $any_attempt++;
-
         $self->debug("localizing funkyftpwise[$url]") if $CPAN::DEBUG;
 
         # Try the most capable first and leave ncftp* for last as it only
@@ -735,8 +706,8 @@ sub hostdlhard {
         my $proxy_vars = $self->_proxy_vars($ro_url);
       DLPRG: for my $f (qw(curl wget lynx ncftpget ncftp)) {
             my $funkyftp = CPAN::HandleConfig->safe_quote($CPAN::Config->{$f});
-            next DLPRG unless defined $funkyftp;
-            next DLPRG if $funkyftp =~ /^\s*$/;
+            next unless defined $funkyftp;
+            next if $funkyftp =~ /^\s*$/;
 
             my($asl_ungz, $asl_gz);
             ($asl_ungz = $aslocal) =~ s/\.gz//;
@@ -787,7 +758,6 @@ $content
                         $CPAN::Frontend->mysleep(1);
                         next DLPRG;
                     }
-                    $some_dl_success++;
                 } else {
                     $CPAN::Frontend->myprint(qq{
 No success, the file that lynx has downloaded is an empty file.
@@ -798,20 +768,13 @@ No success, the file that lynx has downloaded is an empty file.
             if ($wstatus == 0) {
                 if (-s $aslocal) {
                     # Looks good
-                    $some_dl_success++;
                 } elsif ($asl_ungz ne $aslocal) {
                     # test gzip integrity
                     if (eval{CPAN::Tarzip->new($asl_ungz)->gtest}) {
                         # e.g. foo.tar is gzipped --> foo.tar.gz
                         rename $asl_ungz, $aslocal;
-                        $some_dl_success++;
                     } else {
                         eval{CPAN::Tarzip->new($asl_gz)->gzip($asl_ungz)};
-                        if ($@) {
-                            warn "Warning: $@";
-                        } else {
-                            $some_dl_success++;
-                        }
                     }
                 }
                 $ThesiteURL = $ro_url;
@@ -857,15 +820,8 @@ No success, the file that lynx has downloaded is an empty file.
     });
             }
             return if $CPAN::Signal;
-        } # download/transfer programs (DLPRG)
+        } # transfer programs
     } # host
-    return unless $any_attempt;
-    if ($some_dl_success) {
-        $CPAN::Frontend->mywarn("Warning: doesn't seem we had substantial success downloading '$aslocal'. Don't know how to proceed.\n");
-    } else {
-        $CPAN::Frontend->mywarn("Warning: no success downloading '$aslocal'. Giving up on it.\n");
-    }
-    return;
 }
 
 #-> CPAN::FTP::_proxy_vars
